package com.winhc.phoenix.example.aspect;

import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.stereotype.Component;

/**
 * @author: XuJiakai
 * 2020/11/24 17:34
 */
@Slf4j
@Aspect
@Component
public class TimeAspect {

    @Pointcut("@annotation(com.winhc.phoenix.example.aspect.Timer)")
    private void pointcut() {
    }

    @Around("pointcut()")
    public Object around(ProceedingJoinPoint joinPoint) throws Throwable {

        // 获取目标类名称
        String clazzName = joinPoint.getTarget().getClass().getName();

        // 获取目标类方法名称
        String methodName = joinPoint.getSignature().getName();

        long start = System.currentTimeMillis();
//        log.info("{}.{}(): start...", clazzName, methodName);

        // 调用目标方法
        Object result = joinPoint.proceed();

        long time = System.currentTimeMillis() - start;
        log.info("{}.{}(): {} ms", clazzName, methodName, time);
        return result;
    }
}
