package com.winhc.phoenix.example.configuration;

import lombok.extern.slf4j.Slf4j;
import org.apache.http.util.EntityUtils;
import org.elasticsearch.client.Response;
import org.elasticsearch.client.RestClient;
import org.elasticsearch.client.RestHighLevelClient;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author: XuJiakai
 * 2021/8/5 14:40
 */
@Slf4j
public class DynamicElasticSearchClient {
    private final Map<String, RestClient> restClientMap;
    private final Map<String, RestHighLevelClient> restHighLevelClientMap;
    private final String primary;

    private final Map<String, List<String>> indexMap;


    public DynamicElasticSearchClient(Map<String, RestClient> restClientMap, Map<String, RestHighLevelClient> restHighLevelClientMap, String primary) {
        this.restClientMap = restClientMap;
        this.restHighLevelClientMap = restHighLevelClientMap;
        this.primary = primary;
        Map<String, Set<String>> tmpIndexMap = restClientMap.entrySet().stream().collect(Collectors.toMap(e -> e.getKey().replace("_client", ""), e -> {
            RestClient restClient = e.getValue();
            try {
                Response get = restClient.performRequest("get", "_cat/indices?h=index");
                String s = EntityUtils.toString(get.getEntity());
                Set<String> collect = Arrays.stream(s.split("\n")).filter(index -> !index.startsWith(".")).collect(Collectors.toSet());

                Response get2 = restClient.performRequest("get", "_cat/aliases?h=alias");
                String s2 = EntityUtils.toString(get2.getEntity());
                Set<String> collect1 = Arrays.stream(s2.split("\n")).filter(index -> !index.startsWith(".")).collect(Collectors.toSet());

                collect1.addAll(collect);
                return collect1;
            } catch (Exception ex) {
                log.error(ex.getMessage(), e.getKey());
                log.error(ex.getMessage(), ex);
                throw new RuntimeException(ex);
            }
        }, (t1, t2) -> t1));
        indexMap = tmpIndexMap.entrySet().stream().flatMap(e -> {
            String client = e.getKey();
            return e.getValue().stream().map(v -> v + "\001" + client);
        }).collect(Collectors.groupingBy(e -> e.split("\001")[0], Collectors.mapping(e -> e.split("\001")[1], Collectors.toList())));
    }

    public RestClient getRestClient() {
        return getRestClient(primary);
    }

    public RestHighLevelClient getRestHighLevelClient() {
        return getRestHighLevelClient(primary);
    }


    public RestClient getRestClientByIndex(String index) {
        List<String> strings = indexMap.get(index);
        if (strings.size() == 1) {
            return getRestClient(strings.get(0));
        }
        if (strings.contains(primary)) {
            return getRestClient(primary);
        } else {
            throw new RuntimeException("要查询的index在多个集群中都有 ！");
        }
    }

    public RestHighLevelClient getRestHighLevelClientByIndex(String index) {
        List<String> strings = indexMap.get(index);
        if (strings.size() == 1) {
            return getRestHighLevelClient(strings.get(0));
        }
        if (strings.contains(primary)) {
            return getRestHighLevelClient(primary);
        } else {
            throw new RuntimeException("要查询的index在多个集群中都有 ！");
        }
    }


    public RestClient getRestClient(String ds) {
        if (!restClientMap.containsKey(ds + "_client")) {
            throw new RuntimeException(ds + " not fount !");
        }
        return restClientMap.get(ds + "_client");

    }

    public RestHighLevelClient getRestHighLevelClient(String ds) {
        if (!restHighLevelClientMap.containsKey(ds + "_high_client")) {
            throw new RuntimeException(ds + " not fount !");
        }
        return restHighLevelClientMap.get(ds + "_high_client");
    }
}
