package com.winhc.phoenix.example.framework.mongo;

import com.mongodb.client.MongoDatabase;
import lombok.extern.slf4j.Slf4j;
import org.bson.Document;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;
import java.util.function.Consumer;

/**
 * @Author: XuJiakai
 * @Date: 2020/10/21 09:46
 * @Description:
 */
@Slf4j
public class MongoDbFastScan {
    int thread_num = 1;
    int data_num;
    int batchSize = 500;
    int skip = 0;
    String scanTable;
    Consumer<List<Document>> func;
    protected MongoDatabase mongoDatabase;

    public MongoDbFastScan(String scanTable, Consumer<List<Document>> func, MongoDatabase mongoDatabase) {
        this.scanTable = scanTable;
        this.func = func;
        this.mongoDatabase = mongoDatabase;
    }

    public MongoDbFastScan threadNum(int thread_num) {
        this.thread_num = thread_num;
        return this;
    }

    public MongoDbFastScan skip(int skip) {
        this.skip = skip;
        return this;
    }

    public MongoDbFastScan batchSize(int batchSize) {
        this.batchSize = batchSize;
        return this;
    }

    private void scan(int skip, int limit) {
        List<Document> list = new ArrayList<>();
        for (Document document : mongoDatabase.getCollection(scanTable).find().skip(skip).limit(limit).noCursorTimeout(true).batchSize(batchSize)) {
            list.add(document);
            if (list.size() >= batchSize) {
                func.accept(list);
                list.clear();
            }
        }
        if (!list.isEmpty()) {
            func.accept(list);
            list.clear();
        }
        log.info("start:{} end:{} success", skip, skip + limit);
    }

    public void scan() {
        double count = new Long(mongoDatabase.getCollection(scanTable).countDocuments()).doubleValue();
        data_num = new Double(Math.ceil(count / (double) thread_num) * thread_num).intValue();
        System.out.println("data number:" + data_num + " thread num:" + thread_num);
        if (count == 0d) {
            return;
        }
        Vector<Thread> ts = new Vector<>();
        int skipValue = skip % (data_num / thread_num);
        int skipContinue = skip / (data_num / thread_num);

        for (int i = 0; i < thread_num; i++) {
            final int a = i;
            if (a < skipContinue) {
                continue;
            } else {
                Thread t = new Thread(() -> scan(a * (data_num / thread_num) + skipValue, data_num / thread_num - skipValue));
                ts.add(t);
                t.start();
            }
        }

        for (Thread t : ts) {
            try {
                t.join();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
        System.out.println(scanTable + " scan is success");
    }
}
